import argparse
import logging
from pi_pir_webthing.motionsensor_webthing import run_server
from pi_pir_webthing.unit import register, deregister, printlog, list_installed

PACKAGENAME = 'pi_pir_webthing'
ENTRY_POINT = "pir"
DESCRIPTION = "A web connected PIR motion sensor detecting movement running on Raspberry Pi"


def print_info():
    print("usage " + ENTRY_POINT + " --help for command options")
    print("examples")
    for service_info in list_installed(PACKAGENAME):
        port = service_info[1]
        is_active = service_info[2]
        print("sudo " + ENTRY_POINT + " --command log --port " + port)
        print("sudo " + ENTRY_POINT + " --command deregister --port " + port + " --gpio 14")
        if not is_active:
            print("sudo " + ENTRY_POINT + " --command listen --port " + port + " --gpio 14")

def main():
    parser = argparse.ArgumentParser(description=DESCRIPTION)
    parser.add_argument('--command', metavar='command', required=False, type=str, help='the command. Supported commands are: listen (run the webthing service), register (register and starts the webthing service as a systemd unit, deregister (deregisters the systemd unit), log (prints the log)')
    parser.add_argument('--port', metavar='port', required=False, type=int, help='the port of the webthing serivce')
    parser.add_argument('--gpio', metavar='gpio', required=False, type=int, help='the gpio number wired to the device')
    parser.add_argument('--name', metavar='name', required=False, type=str, default="", help='the name')
    args = parser.parse_args()

    if args.command is None:
        print_info()
    elif args.command == 'listen':
        if args.port is None:
            print("--port is mandatory")
        elif args.gpio is None:
            print("--gpio is mandatory")
        else:
            print("running " + PACKAGENAME + "/" + args.name + " on port " + str(args.port) + "/gpio " + str(args.gpio))
            run_server(int(args.port), int(args.gpio), args.name, DESCRIPTION)
    elif args.command == 'register':
        if args.port is None:
            print("--port is mandatory")
        elif args.gpio is None:
            print("--gpio is mandatory")
        else:
            print("register " + PACKAGENAME + "/" + args.name + " on port " + str(args.port) + "/gpio " + str(args.gpio) + " and starting it")
            register(PACKAGENAME, ENTRY_POINT, int(args.port), args.name, int(args.gpio))
    elif args.command == 'deregister':
        if args.port is None:
            print("--port is mandatory")
        else:
            deregister(PACKAGENAME, int(args.port))
    elif args.command == 'log':
        if args.port is None:
            print("--port is mandatory")
        else:
            printlog(PACKAGENAME, int(args.port))
    else:
        print("unsupported command")
        print_info()

        
if __name__ == '__main__':
    logging.basicConfig(format='%(asctime)s %(levelname)-8s %(message)s', level=logging.INFO, datefmt='%Y-%m-%d %H:%M:%S')

    main()




from webthing import (SingleThing, Property, Thing, Value, WebThingServer)
import logging
import RPi.GPIO as GPIO
from datetime import datetime
import tornado.ioloop


class MotionSensor(Thing):

    # regarding capabilities refer https://iot.mozilla.org/schemas
    # there is also another schema registry http://iotschema.org/docs/full.html not used by webthing

    def __init__(self, gpio_number, name, description):
        Thing.__init__(
            self,
            'urn:dev:ops:motionSensor-1',
            'Motion '+ name,
            ['MotionSensor'],
            description
        )

        self.gpio_number = gpio_number
        GPIO.setmode(GPIO.BCM)
        GPIO.setup(self.gpio_number, GPIO.IN)
        GPIO.add_event_detect(self.gpio_number, GPIO.BOTH, callback=self.__update, bouncetime=5)
        self.is_motion = False

        self.motion = Value(False)
        self.add_property(
        Property(self,
                 'motion',
                 self.motion,
                 metadata={
                     '@type': 'MotionProperty',
                     'title': 'motion',
                     "type": "boolean",
                     'description': 'Whether ' + name  + ' motion is detected',
                     'readOnly': True,
                 }))

        self.last_motion = Value(datetime.now().isoformat())
        self.add_property(
            Property(self,
                     'last_motion',
                     self.last_motion,
                     metadata={
                         'title': 'last_motion',
                         "type": "string",
                         'description': 'The date time of last movement',
                         'readOnly': True,
                     }))
        self.ioloop = tornado.ioloop.IOLoop.current()

    def __update(self, channel):
        if GPIO.input(self.gpio_number):
            logging.info("motion detected")
            self.ioloop.add_callback(self.__update_motion_prop, True)
        else:
            self.ioloop.add_callback(self.__update_motion_prop, False)

    def __update_motion_prop(self, is_motion):
        if is_motion:
            self.motion.notify_of_external_update(True)
            self.last_motion.notify_of_external_update(datetime.now().isoformat())
        else:
            self.motion.notify_of_external_update(False)
        logging.info("state updated: " + str(is_motion))


def run_server(port, gpio_number, name, description):
    motion_sensor = MotionSensor(gpio_number, name, description)
    server = WebThingServer(SingleThing(motion_sensor), port=port)
    try:
        logging.info('starting the server')
        server.start()
    except KeyboardInterrupt:
        logging.info('stopping the server')
        server.stop()
        logging.info('done')

